/*  
    next-hack's sd-test.
    Copyright 2017 Nicola Wrachien and Franco Gobbo (next-hack.com)
    This file is part of next-hack's sd-test.

    next-hack's sd-test is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    next-hack's sd-test is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with next-hack's sd-test.  If not, see <http://www.gnu.org/licenses/>.   
*/
#include "ff.h"
FATFS g_fs;
FIL g_file;
extern unsigned char highSpeed;   // global variable that instructs the fatFS to run at low or high speed.
#define REDLED 7
#define GREENLED 6
#define BLUELED 5
#define BLINK_EVERY 32768  // how many bytes we must read between LED blinks
char dummy[512];           // dummy buffer where we will load the data read from the file
void setup() 
{
  pinMode(REDLED, OUTPUT);    
  pinMode(GREENLED, OUTPUT);  
  pinMode(BLUELED, OUTPUT);   
  digitalWrite(REDLED, HIGH);  // RED LED initially on
  digitalWrite(GREENLED, LOW);  // green LED initially off
  digitalWrite(BLUELED, LOW);  // blue LED initially off
  //
  Serial.begin(9600);
  Serial.print(F("Start of Program!\n"));
}
void loop() 
{
  unsigned char res;          // result from the FatFs functions 
  unsigned int bytesRead;     // bytes read by the f_read function
  unsigned long microStart;   // used to keep the time.
  unsigned long totalRead;    // total bytes read
  //
  // Start at low speed mode
  //
  Serial.print(F("Begin low speed test!\n"));    
  for (highSpeed = 0; highSpeed < 2; highSpeed ++)
  {  
    totalRead = 0;
    res = f_mount(&g_fs, "0:", 1);
    if (!res)
        Serial.print(F("f_mount: success!\n"));
    else
    {
      Serial.print(F("f_mount failure! Bye!\n"));
      while(1);   // our journey ends here!
    }
    res = f_open(&g_file, "test.bmp",FA_READ);
    if (!res)
        Serial.print(F("f_open: success!\n"));
    else
    {
      Serial.print(F("f_open failure! Bye!\n"));
      while(1);   // our journey ends here!
    }  
    microStart = micros();
    do
    {
      f_read(&g_file, dummy, sizeof(dummy), &bytesRead);
      if (res)
      {
        Serial.print(F("f_read failure! Does test.bmp exist? Bye!\n"));
        while(1);   // our journey ends here!      
      }
      // Note: since digitalWrite is slow as hell, we change it as seldom as possible. 
      // therefore we check when the state has changed, and in case, we call it.
      if ((totalRead & BLINK_EVERY) != ((totalRead + bytesRead ) & BLINK_EVERY) )
        digitalWrite(GREENLED, !((totalRead + bytesRead) & BLINK_EVERY));     // change the green LED state every BLINK_EVERY bytes    
      totalRead += bytesRead;
    }
    while (bytesRead);
    Serial.print(F("read time taken: "));
    Serial.print(micros() - microStart);
    Serial.print(F(" microseconds\n"));      
    f_close(&g_file);                     // close file
    res = f_mount(NULL, "0:", 1);         // unmount sd
    if (!highSpeed)
    {
      Serial.print(F("Begin high speed (8 MHz) test!\n"));  
      digitalWrite(BLUELED, HIGH);    // turn on blue LED 
    }
  }
  // turn off RED LED and on the green: all went well!
  digitalWrite(REDLED, LOW);  
  digitalWrite(GREENLED, HIGH);    
  Serial.print(F("High speed test successful! Bye!\n"));    
  while (1);    // end
}
